/* shellCommand.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "shellCommand.h"
#include "childPidWatcher.h"

#include <signal.h>
#include <unistd.h>

ShellCommand::
ShellCommand() {
  _pid = 0;
}

ShellCommand::
~ShellCommand() {
  kill();
}

bool ShellCommand::
run(const string &command) {
  cerr << command << "\n";

  // We must block the SIGCHLD signal until we've connected to the
  // watch_pid event.  Otherwise, we could miss the signal (if the
  // child terminates immediately), or worse, enter a race condition
  // with the ChildPidWatcher.

  sigset_t sigchld_set;
  sigemptyset(&sigchld_set);
  sigaddset(&sigchld_set, SIGCHLD);
  sigprocmask(SIG_BLOCK, &sigchld_set, (sigset_t *)NULL);

  _pid = fork();
  if (_pid < 0) {
    cerr << "Unable to fork process\n";
    failure();
    _pid = 0;
    return false;
  }

  if (_pid == 0) {
    // Child.
    sigprocmask(SIG_UNBLOCK, &sigchld_set, (sigset_t *)NULL);
    setpgid(0, 0);
    execl("/bin/sh", "/bin/sh", "-c", command.c_str(), NULL);
    cerr << "Unable to exec " << command << ".\n";
    _exit(1);
  }

  // Parent.
  connect_to_method(ChildPidWatcher::watch_pid(_pid),
		    this, &child_pid_done);

  sigprocmask(SIG_UNBLOCK, &sigchld_set, (sigset_t *)NULL);

  return true;
}

void ShellCommand::
kill() {
  if (_pid > 0) {
    ::kill(_pid, SIGTERM);
  }
}

bool ShellCommand::
is_running() {
  return (_pid > 0);
}


void ShellCommand::
child_pid_done(pid_t pid, bool normal_exit, int exit_status) {
  if (pid != _pid) {
    cerr << "Unexpected child " << pid << " terminated; expected " 
	 << _pid << "\n";
    return;
  }

  _pid = 0;

  if (normal_exit && exit_status == 0) {
    success();
  } else {
    failure();
  }
}
