/* cdDialog.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef CDDIALOG_H
#define CDDIALOG_H

#include "palomaDialog.h"
#include "artistEntry.h"
#include "dataset.h"
#include "tableDataProxy.h"
#include "form.h"
#include "playlistControls.h"
#include "playlistProxy.h"
#include "playlist.h"
#include "pickOne.h"

#include <gtk--.h>
#include <string>
#include <vector>
#include <set>

class CDTrackListRow;
class CDReader;
class CDDialog;

// These are the various choices in the "Performer / Title" combo
// box.
enum CDInterpretNameType {
  IN_title,
  IN_performer_slash_title,
  IN_writer_slash_title,
  IN_performer_hyphen_title,
  IN_writer_hyphen_title,
  IN_performer_dash_title,
  IN_writer_dash_title,
  IN_performer_colon_title,
  IN_writer_colon_title,
  IN_performer_spaces_title,
  IN_writer_spaces_title,
  IN_performer_TITLE,
  IN_writer_TITLE,
  IN_title_slash_performer,
  IN_title_slash_writer,
  IN_title_hyphen_performer,
  IN_title_hyphen_writer,
  IN_title_dash_performer,
  IN_title_dash_writer,
  IN_title_colon_performer,
  IN_title_colon_writer,
  IN_title_spaces_performer,
  IN_title_spaces_writer,
  IN_title_paren_performer,
  IN_title_paren_writer,
  IN_title_bracket_performer,
  IN_title_bracket_writer,
};


// This class contains the data that is kept around for each
// track.  Specifically, it's the TI_songs and TI_songs_extension
// row for each track.  The _load flag also tracks whether the song
// should be loaded from the CD when the dialog is closed.
class CDTrackData {
public:
  CDTrackData();
  TableDataProxy _songs;
  TableDataProxy _songs_extension;

  // This flag indicates whether we are set to load the track into a
  // WAV file when we close the dialog.
  bool _load;
  
  // These are used to manage the "Performer / Title" combo box.
  // They hold the original values of the title and performer etc.,
  // at the time the dialog was created, and until they are
  // specifically changed by the user, so that we can go back and
  // forth between the various choices in the combo box.
  void set_orig_values();
  void interpret_name(CDInterpretNameType type);
  void extract_artist(bool performer, bool artist_first,
		      const string &separator,
		      const string &trail = string());
  void extract_artist_caps(bool performer, bool title_caps);
  void fix_capitalization();
  
  string _orig_name;
  TableDataProxy _orig_performer;
  TableDataProxy _orig_writer;
};

class CDTrackList : public Playlist {
public:
  CDTrackList();

  virtual TableDataProxy get_table_data(int row);
  virtual void fill_dialog_data(int song_row, Dataset &data);
  virtual void dialog_created(int row, PalomaDialog *dialog);
  virtual void dialog_accepted(int row, PalomaDialog *dialog);

  void update_row(int row);
  virtual void update_clist(int row);

  typedef vector<CDTrackData> Tracks;
  Tracks _tracks;
  CDDialog *_cd_dialog;
};

class CDTrackListProxy : public PlaylistProxy {
public:
  CDTrackListProxy(CDTrackList *ptr);
  CDTrackListProxy(const CDTrackListProxy &copy);
  CDTrackListProxy &operator = (const CDTrackListProxy &copy);

  CDTrackData &get_track(int row);
  void set_cd_dialog(CDDialog *dialog);
};

class CDDialog : public PalomaDialog {
public:
  CDDialog(const Dataset &data, int form_id, 
	   const CDTrackListProxy &track_list, 
	   CDReader *cd_reader = (CDReader *)NULL);
  ~CDDialog();

  // The CDDialog becomes the owner of the CDReader pointer.  It will
  // delete it on destruction.
  static CDDialog *make_dialog(const Dataset &data, CDReader *cd_reader);
  static CDDialog *make_dialog(const Dataset &data, int form_id);

  virtual void record_db();

  void check_load_all();
  void menu_same_performer();
  void menu_make_playlist();

private:
  void interpret_name_changed();
  void capitalization_button_clicked();
  void show_performer_clist_clicked();
  void show_writer_clist_clicked();
  void performer_activated();
  void writer_activated();
  void load_all_clicked();
  void load_none_clicked();
  void ok_button_clicked();
  void cancel_button_clicked();

  void layout_window();
  void load_tracks();

  Gtk_ItemFactory_MenuBar _menu;
  Gtk_ObjectHandle<Gtk_MenuBar> _menubar;
  Gtk_VBox _box1;
  Gtk_VBox _box2;
  Gtk_Label _cd_title_label;
  Gtk_Entry _cd_title_entry;
  Gtk_Label _performer_label;
  ArtistEntry _performer_entry;
  Gtk_Label _writer_label;
  ArtistEntry _writer_entry;
  Gtk_ScrolledWindow _track_window;
  Form _form;
  Gtk_CList *_clist;
  Gtk_Table _below_clist;
  PickOne _interpret_name_pick;
  Gtk_Button _capitalization_button;
  Gtk_CheckButton _show_performer_clist_check;
  Gtk_CheckButton _show_writer_clist_check;
  Gtk_Button _load_all;
  Gtk_Label _load_all_label;
  Gtk_Button _load_none;
  PlaylistControls _controls;
  Gtk_Button _ok_button;
  Gtk_Button _cancel_button;

  int _form_id;
  CDReader *_cd_reader;
  CDTrackListProxy _track_list;
  bool _reload_all;
};


#endif

