/* tableDataProxy.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "tableDataProxy.h"
#include "tableData.h"
#include "dataCell.h"

TableDataProxy::
TableDataProxy() {
  _table_id = TI_no_table;
}

TableDataProxy::
TableDataProxy(TableData *ptr) : ReferenceCountProxy(ptr) {
  _table_id = ptr->get_table_id();
}

TableDataProxy::
TableDataProxy(TableID table_id) {
  _table_id = table_id;
}

TableDataProxy::
TableDataProxy(TableID table_id, int primary_key_value) {
  _table_id = table_id;
  assert(_table_id != TI_no_table);

  if (primary_key_value >= 0) {
    set_ptr(TableData::find_table_data(table_id, primary_key_value));
  }
}

TableDataProxy::
TableDataProxy(const TableDataProxy &copy) : 
  ReferenceCountProxy(copy),
  _table_id(copy._table_id)
{
}

TableDataProxy &TableDataProxy::
operator = (const TableDataProxy &copy) {
  ReferenceCountProxy::operator = (copy);
  _table_id = copy._table_id;
  return *this;
}

bool TableDataProxy::
operator == (const TableDataProxy &other) const {
  if (_table_id != other._table_id) {
    return false;
  }
  
  if (get_ptr() == NULL || other.get_ptr() == NULL) {
    // If either table has no data, it matches nothing.
    return false;
  }

  if (get_ptr() == other.get_ptr()) {
    // If somehow the pointers match up, the proxies are of course equal.
    return true;
  }

  // Now they only match up if the primary keys match.
  return (get_primary_key() >=0 &&
	  get_primary_key() == other.get_primary_key());
}

// Returns true if there actually is a TableData associated with this
// proxy, or false otherwise.  A TableData will be associated if it
// was loaded via the constructor or if it was implicitly created when
// some non-const member function was called at some point.
bool TableDataProxy::
has_table() const {
  return (ReferenceCountProxy::get_ptr() != NULL);
}

TableID TableDataProxy::
get_table_id() const {
  return _table_id;
}

bool TableDataProxy::
has_fields() const {
  return get_const_ptr()->has_fields();
}

void TableDataProxy::
set_primary_key(int primary_key_value, bool new_data) {
  get_mod_ptr()->set_primary_key(primary_key_value, new_data);
}

int TableDataProxy::
get_primary_key() const {
  return get_const_ptr()->get_primary_key();
}

bool TableDataProxy::
is_new_data() const {
  return get_const_ptr()->is_new_data();
}

int TableDataProxy::
record_db() {
  return get_mod_ptr()->record_db();
}

int TableDataProxy::
record_db(int primary_key_value) {
  return get_mod_ptr()->record_db(primary_key_value);
}


DataCell &TableDataProxy::
operator [] (int field_id) {
  return get_mod_ptr()->operator[](field_id);
}

const DataCell TableDataProxy::
operator [] (int field_id) const {
  return get_const_ptr()->operator[](field_id);
}

void TableDataProxy::
set_db_value(int field_id, const DataCell &value) {
  get_mod_ptr()->set_db_value(field_id, value);
}

const DataCell TableDataProxy::
get_db_value(int field_id) const {
  return get_const_ptr()->get_db_value(field_id);
}

TableData *TableDataProxy::
get_mod_ptr() {
  // A TableDataProxy is never empty.  If we ask for its pointer and
  // the pointer is unassigned, that means we meant to get an empty
  // TableData.

  TableData *ptr = (TableData *)ReferenceCountProxy::get_ptr();
  if (ptr == NULL) {
    assert(_table_id != TI_no_table);

    ptr = new TableData(_table_id);
    ((TableDataProxy *)this)->set_ptr(ptr);
  }

  return ptr;
}

const TableData *TableDataProxy::
get_const_ptr() const {
  // See above.  In the case of a const pointer, though, we don't want
  // to create a new TableData instance if we don't have one; instead,
  // we'll just return a local pointer.

  TableData *ptr = (TableData *)ReferenceCountProxy::get_ptr();
  if (ptr == NULL) {
    assert(_table_id != TI_no_table);
    
    // We do a bit of finagling here so we can safely return a
    // pointer.  We can't just make a local TableData object, since
    // that would go out of scope when the function returned, and
    // creating a static TableData object seems to be bad for GTK
    // reasons.  Instead, we'll create a static pointer, which we
    // dynamically allocate exactly once, and never free.

    static TableData *static_ptr = NULL;
    if (static_ptr == NULL) {
      static_ptr = new TableData(_table_id);
    }

    (*static_ptr) = TableData(_table_id);
    return static_ptr;
  }

  return ptr;
}

void TableDataProxy::
output(ostream &out) const {
  if (_table_id == TI_no_table) {
    out << "no table";
  } else if (ReferenceCountProxy::get_ptr() == NULL) {
    out << "table " << _table_id << ", no table";
  } else {
    out << *get_const_ptr();
  }
}
